from sklearn.datasets import load_boston
from easy_sklearn import EasySklearnRegressor
import warnings
warnings.filterwarnings("ignore")
ds = load_boston()
X = ds.data
y = ds.target
c = ds.feature_names

esc = EasySklearnRegressor()
X_train, X_test, y_train, y_test = esc.split_data(X, y)
print(X_train.shape, X_test.shape)
esc.set()
esc.fit(X_train, y_train)
esc.score(X_test, y_test)
esc.plot_learning_curve()



from sklearn.datasets import load_iris
from easy_sklearn import EasySklearnClassifier

ds = load_iris()
X = ds.data
y = ds.target
c = ds.feature_names

esc = EasySklearnClassifier()
X_train, X_test, y_train, y_test = esc.split_data(X, y)
esc.set()
esc.fit(X_train, y_train)
esc.score(X_test, y_test)
esc.plot_learning_curve()

clf = esc.best_model_
print(clf)



# -*- coding: utf-8 -*-
import matplotlib.pyplot as plt
import pandas as pd
import time
import utils
from sklearn.model_selection import train_test_split
import copy
from sklearn import preprocessing


class EasySklearn(object):

    def __init__(self):
        self.es_models = {}
        self.es_scalers = {}
        self.X_train_ = []
        self.best_model_ = None
        self.best_scaler_ = None
        self.scores = pd.DataFrame(columns=('model', 'scaler', 'train_score', 'valid_score', 'time'))

    def split_data(self, X, y, test_size=0.2, random_state=1):
        return train_test_split(X, y, test_size=test_size, random_state=random_state)

    @property
    def default_scalers_(self):
        return {
            'maxabs': preprocessing.MaxAbsScaler(),
            'robust': preprocessing.RobustScaler(),
            'scale': preprocessing.StandardScaler(),
            'norm': preprocessing.Normalizer(),
            'qt': preprocessing.QuantileTransformer(),
            'minmax': preprocessing.MinMaxScaler(),
        }

    @property
    def default_scalers_name_(self):
        return [scaler for scaler in self.default_scalers_]

    @property
    def default_models_(self):
        return {}

    @property
    def default_models_name_(self):
        return {}

    def set(self, models='', scalers=''):
        _models, _scalers = {}, {}
        if models is '':
            _models = self.default_models_
        else:
            _models[models] = self.default_models_[models]
        self.es_models = _models

        if scalers is '':
            _scalers = self.default_scalers_
        else:
            _scalers[scalers] = self.default_scalers_[scalers]
        self.es_scalers = _scalers

    def fit(self, X, y):
        print('begin train:------------------------------')
        print('traing data shape', X.shape)
        self.X_train_, self.y_train_ = X, y
        X_train, X_valid, y_train, y_valid = self.split_data(X, y)
        i, best_score = 0, 0
        for scaler_name in self.es_scalers:
            print('training scaler:', scaler_name)
            scaler = self.es_scalers[scaler_name].fit(X_train)
            X_train_trans = scaler.transform(X_train)
            X_valid_trans = scaler.transform(X_valid)
            for model in self.es_models:
                start_time = time.time()
                clf = self.es_models[model]['clf']
                clf.fit(X_train_trans, y_train)
                train_score = clf.score(X_train_trans, y_train)
                valid_score = clf.score(X_valid_trans, y_valid)
                self.scores.loc[i] = [model, scaler_name, train_score, valid_score, time.time() - start_time]
                if valid_score > best_score:
                    print('find best model', scaler_name, model)
                    self.best_model_ = copy.deepcopy(clf)
                    self.best_scaler_ = copy.deepcopy(scaler)
                    best_score = valid_score
                i += 1

        self.scores.sort_values(by=['valid_score'], ascending=0, inplace=True)
        self.scores.reset_index(inplace=True)
        del self.scores['index']
        print('\ntrain result:------------------------------')
        print(self.scores.head(10))
        print('\n group by model')
        print(self.scores.groupby('model').mean().sort_values(by=['valid_score'], ascending=0))
        print('\n group by scaler')
        print(self.scores.groupby('scaler').mean().sort_values(by=['valid_score'], ascending=0))

    def getModel(self, model, scaler):
        if model is '':
            _model = self.best_model_
            _model_name = self.scores.loc[0, 'model']
        else:
            _model = self.es_models[model]
            _model_name = model

        if scaler == '':
            _scaler = self.best_scaler_
            _scaler_name = self.scores.loc[0, 'scaler']
        else:
            _scaler = self.es_scalers[scaler]
            _scaler_name = scaler
        return _model, _model_name, _scaler, _scaler_name

    def score(self, X, y, model='', scaler=''):
        print('\ntest result:------------------------------')
        _model, _model_name, _scaler, _scaler_name = self.getModel(model, scaler)
        score = _model.score(_scaler.transform(X), y)
        print('test data shape:', X.shape, ' test with:', _model_name, _scaler_name)
        print(' score:', score)
        return score

    def plot_learning_curve(self, model='', scaler=''):
        print('\nplot learning curve:------------------------------')
        _model, _model_name, _scaler, _scaler_name = self.getModel(model, scaler)
        title = '%s-%s' % (_model_name, _scaler_name)
        X_train_trans = _scaler.transform(self.X_train_)
        utils.plot_learning_curve(_model, X_train_trans, self.y_train_, title=title)



# -*- coding: utf-8 -*-
import numpy as np
import pandas as pd

def plot_learning_curve(estimator, X, y, title='', ylim=(0.0, 1.01), cv=None,
                        n_jobs=-1, train_sizes=np.linspace(.1, 1.0, 5)):
    """
    画出data在某模型上的learning curve.
    参数解释
    ----------
    estimator : 你用的分类器。
    title : 表格的标题。
    X : 输入的feature，numpy类型
    y : 输入的target vector
    ylim : tuple格式的(ymin, ymax), 设定图像中纵坐标的最低点和最高点
    cv : 做cross-validation的时候，数据分成的份数，其中一份作为cv集，其余n-1份作为training(默认为3份)
    n_jobs : 并行的的任务数(默认1)
    """
    from sklearn.model_selection import learning_curve
    import matplotlib.pyplot as plt
    from sklearn.model_selection import ShuffleSplit
    # plt.figure()
    plt.title('%s Learning curve ' % (title))
    if ylim is not None:
        plt.ylim(*ylim)
    if cv is None:
        ShuffleSplit(n_splits=10, test_size=0.01, random_state=0)

    plt.xlabel("Training examples")
    plt.ylabel("Score")
    train_sizes, train_scores, test_scores = learning_curve(
        estimator, X, y, cv=cv, n_jobs=n_jobs, train_sizes=train_sizes)
    train_scores_mean = np.mean(train_scores, axis=1)
    train_scores_std = np.std(train_scores, axis=1)
    test_scores_mean = np.mean(test_scores, axis=1)
    test_scores_std = np.std(test_scores, axis=1)
    plt.grid()

    plt.fill_between(train_sizes, train_scores_mean - train_scores_std,
                     train_scores_mean + train_scores_std, alpha=0.1,
                     color="r")
    plt.fill_between(train_sizes, test_scores_mean - test_scores_std,
                     test_scores_mean + test_scores_std, alpha=0.1, color="g")
    plt.plot(train_sizes, train_scores_mean, 'o-', color="r",
             label="Training score")
    plt.plot(train_sizes, test_scores_mean, 'o-', color="g",
             label="Cross-validation score")

    plt.legend(loc="best")
    # plt.draw()
    plt.show()

    midpoint = ((train_scores_mean[-1] + train_scores_std[-1]) + (test_scores_mean[-1] - test_scores_std[-1])) / 2
    diff = (train_scores_mean[-1] + train_scores_std[-1]) - (test_scores_mean[-1] - test_scores_std[-1])
    return midpoint, diff


def feature_selection(feature_matrix, missing_threshold=90, correlation_threshold=0.95):
    """Feature selection for a dataframe."""

    feature_matrix = pd.get_dummies(feature_matrix)
    n_features_start = feature_matrix.shape[1]
    print('Original shape: ', feature_matrix.shape)

    # Find missing and percentage
    missing = pd.DataFrame(feature_matrix.isnull().sum())
    missing['percent'] = 100 * (missing[0] / feature_matrix.shape[0])
    missing.sort_values('percent', ascending=False, inplace=True)

    # Missing above threshold
    missing_cols = list(missing[missing['percent'] > missing_threshold].index)
    n_missing_cols = len(missing_cols)

    # Remove missing columns
    feature_matrix = feature_matrix[[x for x in feature_matrix if x not in missing_cols]]
    print('{} missing columns with threshold: {}.'.format(n_missing_cols,
                                                          missing_threshold))

    # Zero variance
    unique_counts = pd.DataFrame(feature_matrix.nunique()).sort_values(0, ascending=True)
    zero_variance_cols = list(unique_counts[unique_counts[0] == 1].index)
    n_zero_variance_cols = len(zero_variance_cols)

    # Remove zero variance columns
    feature_matrix = feature_matrix[[x for x in feature_matrix if x not in zero_variance_cols]]
    print('{} zero variance columns.'.format(n_zero_variance_cols))

    # Correlations
    corr_matrix = feature_matrix.corr()

    # Extract the upper triangle of the correlation matrix
    upper = corr_matrix.where(np.triu(np.ones(corr_matrix.shape), k=1).astype(np.bool))

    # Select the features with correlations above the threshold
    # Need to use the absolute value
    to_drop = [column for column in upper.columns if any(upper[column].abs() > correlation_threshold)]

    n_collinear = len(to_drop)

    feature_matrix = feature_matrix[[x for x in feature_matrix if x not in to_drop]]
    print('{} collinear columns removed with threshold: {}.'.format(n_collinear,
                                                                    correlation_threshold))

    total_removed = n_missing_cols + n_zero_variance_cols + n_collinear

    print('Total columns removed: ', total_removed)
    print('Shape after feature selection: {}.'.format(feature_matrix.shape))
    return feature_matrix



# -*- coding: utf-8 -*-
from easy_base import EasySklearn
from sklearn.ensemble import BaggingRegressor
from sklearn.pipeline import Pipeline
from sklearn.preprocessing import StandardScaler, PolynomialFeatures
from sklearn import tree, ensemble, linear_model, svm, neighbors


class EasySklearnRegressor(EasySklearn):
    def __init__(self):
        EasySklearn.__init__(self)

    @property
    def default_models_(self):
        return {
            'Tree': {'clf': tree.DecisionTreeRegressor(),
                     'param': {}},
            'GBDT': {'clf': ensemble.GradientBoostingRegressor(),
                     'param': {
                         'learning_rate': [0.1],
                         'max_depth': [4, 6],
                         'alpha': [0.9],
                         'max_leaf_nodes': [10, 20],
                         'min_samples_split': [2, 4]
                     }},
            'Lin': {'clf': linear_model.LinearRegression(),
                    'param': {
                        'fit_intercept': [True, False],
                        'normalize': [True, False]
                    }},
            'Ridge': {'clf': linear_model.Ridge(),
                      'param': {}},
            'Lasso': {'clf': linear_model.Lasso(),
                      'param': {}},
            'ElasN': {'clf': linear_model.ElasticNet(),
                      'param': {}},
            'Lars': {'clf': linear_model.Lars(),
                     'param': {}},
            'Bayers': {'clf': linear_model.BayesianRidge(),
                       'param': {}},
            'Poly2': {'clf': Pipeline([('poly', PolynomialFeatures(degree=2)),
                                       ('std_scaler', StandardScaler()),
                                       ('line_reg', linear_model.LinearRegression())
                                       ]),
                      'param': {}},
            'SGD': {'clf': linear_model.SGDRegressor(),
                    'param': {}},
            'SVM': {'clf': svm.SVR(kernel='rbf', C=1.0, epsilon=1),
                    'param': {}},
            'Knn': {'clf': neighbors.KNeighborsRegressor(),
                    'param': {}},
            'RF': {'clf': ensemble.RandomForestRegressor(),
                   'param':
                       {'n_estimators': [3, 10, 30],
                        'bootstrap': [True, False],
                        'max_features': [3, 7]
                        }},
            'ADA': {'clf': ensemble.AdaBoostRegressor(n_estimators=100),
                    'param': {}},
            'BAG': {'clf': BaggingRegressor(bootstrap=True),
                    'param': {'n_estimators': [50, 100, 200]}},
            'ET': {'clf': tree.ExtraTreeRegressor(),
                   'param': {}},
        }

    @property
    def default_models_name_(self):
        return [model for model in self.default_models_]



# -*- coding: utf-8 -*-
from easy_base import EasySklearn
from sklearn.svm import SVC
from sklearn.ensemble import GradientBoostingClassifier
from sklearn import tree
from sklearn.ensemble import RandomForestClassifier, BaggingClassifier
from sklearn.linear_model import LogisticRegression
from sklearn.neighbors import KNeighborsClassifier
from sklearn.neural_network import MLPClassifier
from sklearn.multiclass import OneVsRestClassifier


class EasySklearnClassifier(EasySklearn):
    def __init__(self):
        EasySklearn.__init__(self)

    @property
    def default_models_(self):
        return {
            'SVC': {'clf': SVC(kernel='rbf', probability=False, random_state=1),
                    'param': {}},
            'ORSVC': {'clf': OneVsRestClassifier(SVC(kernel='rbf', probability=False, random_state=1)),
                      'param': {}},
            'GBDT': {'clf': GradientBoostingClassifier(random_state=1),
                     'param': {'n_estimators': [70, 100], 'max_depth': [7], 'subsample': [0.7, 0.9]}},
            'Tree': {'clf': tree.DecisionTreeClassifier(random_state=1),
                     'param': {}},
            'RF': {'clf': RandomForestClassifier(random_state=1),
                   'param': {'n_estimators': [50, 100, 150]}},
            'LR2': {'clf': LogisticRegression(penalty='l2'),
                    'param': {}},
            'KN': {'clf': KNeighborsClassifier(),
                   'param': {}},
            'Bag': {'clf': BaggingClassifier(random_state=1),
                    'param': {}},
            'MLP': {'clf': MLPClassifier(max_iter=1000, random_state=1),
                    'param': {}}
        }

    @property
    def default_models_name_(self):
        return [model for model in self.default_models_]



#!/usr/bin/env python
# -*- coding:utf-8 -*-

#############################################
# File Name: setup.py
# Author: Ruifeng96150
# Mail: ruifeng96150@163.com
# Created Time:  2018-10-21 19:17:34
#############################################


from setuptools import setup, find_packages

setup(
    name="easy_sklearn",
    version="0.1.1",
    keywords=("sklearn", "python", "machine learning"),
    description="This is a python library base on sklearn",
    long_description="This is a python library, which can be easier to build sklearn classification and regressor models.",
    license="MIT Licence",

    url="https://github.com/ruifeng96150/easy_sklearn",
    author="ruifeng96150",
    author_email="ruifeng96150@163.com",
    classifiers=[
        'Development Status :: 4 - Beta',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Developers',
    ],

    packages=find_packages(),
    include_package_data=True,
    platforms="any",
    install_requires=['numpy', 'pandas', 'sklearn']
)
